@@ -11,16 +11,16 @@
 #define __FDT_IPI_H__
 
 #include <sbi/sbi_types.h>
+#include <sbi_utils/fdt/fdt_driver.h>
 
-struct fdt_ipi {
-	const struct fdt_match *match_table;
-	int (*cold_init)(void *fdt, int nodeoff, const struct fdt_match *match);
-	int (*warm_init)(void);
-	void (*exit)(void);
-};
+#ifdef CONFIG_FDT_IPI
 
-void fdt_ipi_exit(void);
+int fdt_ipi_init(void);
 
-int fdt_ipi_init(bool cold_boot);
+#else
+
+static inline int fdt_ipi_init(void) { return 0; }
+
+#endif
 
 #endif
