@@ -12,15 +12,19 @@
 
 #include <sbi/sbi_types.h>
 
+#ifdef CONFIG_FDT_IRQCHIP
+
 struct fdt_irqchip {
 	const struct fdt_match *match_table;
-	int (*cold_init)(void *fdt, int nodeoff, const struct fdt_match *match);
-	int (*warm_init)(void);
-	void (*exit)(void);
+	int (*cold_init)(const void *fdt, int nodeoff, const struct fdt_match *match);
 };
 
-void fdt_irqchip_exit(void);
+int fdt_irqchip_init(void);
 
-int fdt_irqchip_init(bool cold_boot);
+#else
+
+static inline int fdt_irqchip_init(void) { return 0; }
+
+#endif
 
 #endif
