@@ -11,10 +11,20 @@
 #include <sbi_utils/fdt/fdt_helper.h>
 #include <sbi_utils/sys/htif.h>
 
-static int htif_reset_init(void *fdt, int nodeoff,
+static int htif_reset_init(const void *fdt, int nodeoff,
 			   const struct fdt_match *match)
 {
-	return htif_system_reset_init();
+	bool custom = false;
+	uint64_t fromhost_addr = 0, tohost_addr = 0;
+
+	if (!fdt_get_node_addr_size(fdt, nodeoff, 0, &fromhost_addr, NULL)) {
+		custom = true;
+		tohost_addr = fromhost_addr + sizeof(uint64_t);
+	}
+
+	fdt_get_node_addr_size(fdt, nodeoff, 1, &tohost_addr, NULL);
+
+	return htif_system_reset_init(custom, fromhost_addr, tohost_addr);
 }
 
 static const struct fdt_match htif_reset_match[] = {
@@ -22,7 +32,7 @@ static const struct fdt_match htif_reset_match[] = {
 	{ },
 };
 
-struct fdt_reset fdt_reset_htif = {
+const struct fdt_driver fdt_reset_htif = {
 	.match_table = htif_reset_match,
 	.init = htif_reset_init
 };
