@@ -173,7 +173,7 @@ static struct sbi_system_reset_device da9063_reset_i2c = {
 	.system_reset = da9063_system_reset
 };
 
-static int da9063_reset_init(void *fdt, int nodeoff,
+static int da9063_reset_init(const void *fdt, int nodeoff,
 			     const struct fdt_match *match)
 {
 	int rc, i2c_bus;
@@ -209,11 +209,16 @@ static const struct fdt_match da9063_reset_match[] = {
 	{ },
 };
 
-struct fdt_reset fdt_reset_da9063 = {
+const struct fdt_driver fdt_reset_da9063 = {
 	.match_table = da9063_reset_match,
 	.init = da9063_reset_init,
 };
 
+static const struct fdt_driver *const sifive_fu740_reset_drivers[] = {
+	&fdt_reset_da9063,
+	NULL
+};
+
 static u64 sifive_fu740_tlbr_flush_limit(const struct fdt_match *match)
 {
 	/*
@@ -226,14 +231,13 @@ static u64 sifive_fu740_tlbr_flush_limit(const struct fdt_match *match)
 	return 0;
 }
 
-static int sifive_fu740_final_init(bool cold_boot,
+static int sifive_fu740_final_init(bool cold_boot, void *fdt,
 				   const struct fdt_match *match)
 {
 	int rc;
-	void *fdt = fdt_get_address();
 
 	if (cold_boot) {
-		rc = fdt_reset_driver_init(fdt, &fdt_reset_da9063);
+		rc = fdt_driver_init_one(fdt, sifive_fu740_reset_drivers);
 		if (rc)
 			sbi_printf("%s: failed to find da9063 for reset\n",
 				   __func__);
